/*
  Name: spi_asm.asm
  Copyright: (c) Kai Scheddin
  Author: Kai Scheddin
  Date: 02.04.07 16:45
  Description: serial pehripherial interface controll n init asm for ATMEGA32
*/



/***** SPI **************************/

SPDR=0x0f
SPSR=0x0e
SPCR=0x0d
DDR_SPI=0x17
DD_SS=4
DD_MOSI=5
DD_MISO=6
DD_SCK=7

/* SPDR - SPI Data Register*/
SPDR0	= 0	/* SPI Data Register bit 0*/
SPDR1	= 1	/* SPI Data Register bit 1*/
SPDR2	= 2	/* SPI Data Register bit 2*/
SPDR3	= 3	/* SPI Data Register bit 3*/
SPDR4	= 4	/* SPI Data Register bit 4*/
SPDR5	= 5	/* SPI Data Register bit 5*/
SPDR6	= 6	/* SPI Data Register bit 6*/
SPDR7	= 7	/* SPI Data Register bit 7*/

/* SPSR - SPI Status Register*/
SPI2X=0	/* Double SPI Speed Bit*/

WCOL=6	/* Write Collision Flag*/
SPIF=7	/* SPI Interrupt Flag*/

/* SPCR - SPI Control Register*/
SPR0=0	 /*SPI Clock Rate Select 0*/
SPR1=1	 /*SPI Clock Rate Select 1*/
CPHA=2	 /*Clock Phase*/
CPOL=3	 /*Clock polarity*/
MSTR=4	 /*Master/Slave Select*/
DORD=5	 /*Data Order*/
SPE=6	     /*SPI Enable*/
SPIE=7	 /*SPI Interrupt Enable*/



.global spi_clock_set
.func spi_clock_set
in r16,SPCR
in r17,SPSR

cpi r24,2
breq 1f
cpi r24,4
breq 2f
cpi r24,8
breq 3f
cpi r24,16
breq 4f
cpi r24,32
breq 5f
cpi r24,64
breq 6f
cpi r24,128
breq 7f
brne 9f

1:
andi r16,0xfc /*0b11111100*/
ori r17,0x01
rjmp 8f
2:
andi r16,0xfc /*0b11111100*/
andi r17,0xfe
rjmp 8f
3:
ori r16,0x01
andi r16,0xfd /*0b11111101*/
ori r17,0x01
rjmp 8f
4:
ori r16,0x01
andi r16,0xfd /*0b11111101*/
andi r17,0xfe
rjmp 8f
5:
andi r16,0xfe
ori r16,0x02 
ori r17,0x01
rjmp 8f
6:
ori r16,0x03 
ori r17,0x01
rjmp 8f
7:
ori r16,0x03
andi r17,0xfe
rjmp 8f
8:
out SPCR,r16
out SPSR,r17
9:
ret
.endfunc

.global spi_set_master
.func spi_set_master
in r16,SPCR
ori r16,8
out SPCR,r16
ret
.endfunc

.global spi_set_slave
.func spi_set_slave
in r16,SPCR
andi r16,0xf7
out SPCR,r16
ret
.endfunc

.global spi_set_endian
.func spi_set_endian
cpi r24,0x02
brge 12f

in r16,SPCR
cpi r24,0x01
breq 10f
andi r16,0xef
rjmp 11f:
10:
ori r16,0x10
11:
out SPCR,r16	
12:
ret
.endfunc

.global spi_set_spcr
.func spi_set_spcr
out SPCR,r24
ret
.endfunc
.global spi_set_spsr
.func spi_set_spsr
out SPSR,r24
ret
.endfunc
.global spi_set_spdr
.func spi_set_spdr
out spdr,r24
ret
.endfunc
.global spi_get_spcr
.func spi_get_spcr
in r24,spcr
.endfunc
.global spi_get_spsr
.func spi_get_spsr
in r24,SPSR
ret
.endfunc
.global spi_get_spdr
.func spi_get_spdr
in r24,SPDR
ret
.endfunc
.global spi_sendbyte
.func spi_sendbyte
13:
/*using sbrc instead of sbic causes error's by assembling*/
in r16,SPSR
sbrc r16,SPIF
rjmp 13b
out SPDR,r24
14:
in r16,SPSR
sbrs r16,SPIF
rjmp 14b
ret
.endfunc



.global spi_standart_init
.func spi_standart_init
spi_init_m:
/* Set MOSI and SCK output, all others input (also DD_SS=PB4)*/
ldi r17,(1<<DD_MOSI)|(1<<DD_SCK)
out DDR_SPI,r17

//SPI init:
	// - master mode
	// - positive clock phase
	// - msb first
	// - maximum SPI speed (fosc/2)
	// - enable spi
	SPCR = (0<<CPOL)|(1<<MSTR)|(0<<DORD)|(0<<SPR1)|(0<<SPR0)|(1<<SPE);
	SPSR = (1<<SPI2X);



/*Enable SPI, Master, set clock rate fck/16*/
ldi r17,(1<<SPE)|(1<<MSTR)|(1<<SPR0)
out SPCR,r17
ret
.endfunc

.global spi_sendbyte
.func spi_sendbyte
spi_sendbyte:
/* Start transmission of data (r16)*/
out SPDR,r16
1:
/* Wait for transmission complete*/
sbis SPSR,SPIF
rjmp 1b
ret
.endfunc




.end
